#include <iostream>

extern int writeScores( int turns );
extern void getScores( int i, char *name, int &score );

using namespace std;

int main()
{
   char name[9];
   int score, i;

   for( i = 0; i < 3; i++ ) {
      getScores( i, name, score );
      cout << name << ' ' << score << endl;
   }
   
   writeScores( 7 );
   return(0);
}
